/**
 * A simple implementation of a pool for Formula memory management. Whenever you need a
 * pointer to a Formula, call getFormula(). When you are finished with all currently-used
 * Formulas, call freeAllFormulas().
 *
 * Kevin Gimpel
 */
#ifndef FORMULA_POOL_H_
#define FORMULA_POOL_H_

#include <list>
#include <set>

class Formula;

class FormulaPool {
public:
	void initialize(unsigned int capacity);
	Formula* getFormula();
	void freeAllFormulas();
	void resetAllFormulaComputedFlags();
	int getNumberOfFormulasUsed() const;
private:
	void createNewFormula();
	// If using C++11, should use forward_list instead of list below.
	list<Formula> m_formulas;
	vector<Formula*> m_formulaPointers;
	// m_current holds the index of the next Formula pointer to return.
	unsigned int m_current;
};

#endif
