#include <iostream>
#include "../fdag/model.h"

using namespace std;

int main(int argc, char** argv) {
	// Change following to true to do finite difference checking of gradients.
	bool doFiniteDifferenceChecking = true;

	// Create a model with initial Formula capacity 100.
	Model model(100);
	// Create a parameter called 'a' with initial value 5.0.
	PARAM_ID parameterA = model.initializeNewParameter(5.0);

	// Create an FDAG for 2a+3.

	// Create a times node for 2a.
	Formula* times = model.getFormulaObject(TIMES);
	// Add constant 2 as the first child of the times node.
	times->addChild(model.getConstantFormulaObject(2.0));
	// Add parameter a as the second child of the times node.
	times->addChild(model.getCachedParameterFormula(parameterA));

	// Create a plus node for the root node of the FDAG.
	Formula* rootSum = model.getFormulaObject(PLUS);
	// Add the times node as the first child of the plus node.
	rootSum->addChild(times);
	// Add constant 3.0 as the second child of the plus node.
	rootSum->addChild(model.getConstantFormulaObject(3.0));

	// Print the FDAG before calling evaluate.
	cout << "Printing FDAG before evaluation:" << endl;
	rootSum->printFormula();
	// Compute the value and derivatives of the FDAG.
	double value = model.evalAndComputeDerivatives(rootSum, 0, doFiniteDifferenceChecking);
	cout << endl << "Computed value = " << value << endl << endl;
	// Print the FDAG after calling evaluate.
	cout << "Printing FDAG after evaluation:" << endl;
	rootSum->printFormula();
	cout << endl;
	return 0;
}
