#include <iostream>
//#include <stdlib.h>
#include <sys/time.h>
#include "../../utils/cmdparams.h"
#include "../../utils/utils.h"
#include "segmclassifier.h"

int main(int argc, char** argv) {
    ParamsArray params(argc, argv);
    params.setDescription("Runs a speech segment classifier, considering several neural architecures using fdag.");

    params.addConstraint("train_file",
    		"file containing training data, where each line contains the segment label followed by a space and then the "
    		"space-separated sequence of input features for the segment", false); // not optional
    params.addConstraint("dev_file",
    		"file containing dev data to compute accuracy for, same format as train_file", false); // not optional
    params.addConstraint("test_file",
    		"file containing test data to compute accuracy for, same format as train_file", false); // not optional
    params.addConstraint("dev_pred_file_prefix",
    		"prefix of filenames in which to output dev data predicted labels, one segment per line", false); // not optional
    params.addConstraint("test_pred_file_prefix",
    		"prefix of filenames in which to output test data predicted labels, one segment per line", false); // not optional
    params.addConstraint("phone_set_file",
    		"file containing phone label set, one phone per line", false); // not optional
    //params.addConstraint("cv",
    	//	"if > 0, will do CV on train_file instead of testing on dev/test; this is the number of folds to use with CV (default: 0)", true); // optional
    //params.addConstraint("cv_only_do_fold",
    	//	"if >= 0 and cv > 0, will only do the given CV fold; this can range from 0 to cv - 1 (default: -1)", true); // optional

    params.addConstraint("stochastic_pooling",
    		"if true, will do stochastic pooling instead of max pooling (default: false)", true); // optional
    params.addConstraint("l2_norm_constraint",
    		"the s value representing desired l2 norm of weights; after updating, will scale weights such that their norm equals "
    		"this value. pass a value <=0 if no constraining is desired (default: -1.0)", true); // optional
    params.addConstraint("forward_lstms", "configuration of forward LSTMs to train; in format \"number:layer_widths.number:layer_widths...\", where \"number\" "
    		"is the number of forward LSTMs to train that have the following sequence of hidden layer widths. E.g., to train one forward LSTM with "
    		"one hidden layer of width 10 and 2 forward LSTMs with one hidden layer of width 5 and a second hidden layer of width 7, pass \"1:10.2:5,7\". "
    		"Pass \"0\" if no forward LSTMs are desired. (default: 1:10)", true); // optional
    params.addConstraint("backward_lstms", "configuration of backward LSTMs to train, same format as forward_lstms (default: 0)", true); // optional
    params.addConstraint("forward_grus", "configuration of forward GRUs to train, same format as forward_lstms (default: 0)", true); // optional
    params.addConstraint("backward_grus", "configuration of backward GRUs to train, same format as forward_lstms (default: 0)", true); // optional

    params.addConstraint("algorithm",
    		"optimization algorithm; can be \"sgd_momentum\", \"adagrad\", or \"adadelta\" (case insensitive) (default: adadelta)", true); // optional
    params.addConstraint("stepsize",
    		"stepsize for AdaGrad (default: 0.1)", true); // optional
    params.addConstraint("minibatch_size",
    		"mini-batch size for AdaGrad (default: 10)", true); // optional
    params.addConstraint("num_epochs",
    		"number of epochs of learning (default: 3)", true); // optional
    params.addConstraint("softmax_dropout_rate", "probability of setting a unit to zero in the final vector used in the softmax classifier; use a value <= 0 for no dropout (default: 0.5)", true); // optional
    params.addConstraint("input_dropout_rate", "probability of setting a unit to zero in the transition from the input word vector to the first hidden vector; use a value <= 0 for no dropout (default: 0.5)", true); // optional
    params.addConstraint("hidden_dropout_rate", "probability of setting a unit to zero in subsequent transitions from hidden vectors to hidden vectors; use a value <= 0 for no dropout (default: 0.5)", true); // optional
    params.addConstraint("decode_every_k", "number of examples to decode after processing, if train data is bigger than 11k examples (default: 5000)", true); // optional
    params.addConstraint("gradient_norm_threshold", "max gradient norm (\"threshold\" from Pascanu et al paper; use a value <= 0 for no gradient rescaling (default: 1.0)", true); // optional
    params.addConstraint("use_cost", "if true, will use cost-sensitive cross-entropy (default: false)", true); // optional
    params.addConstraint("use_coarse_cost", "if true, will use coarse tags when computing cost-sensitive cross-entropy (default: false)", true); // optional

   	if (!params.runConstraints("run_segmclassify")) return 0;

	// Read in command-line parameters.
	string trainFilename = params.asString("train_file");
	string devFilename = params.asString("dev_file");
	string testFilename = params.asString("test_file");
	string devPredFilenamePrefix = params.asString("dev_pred_file_prefix", "");
	string testPredFilenamePrefix = params.asString("test_pred_file_prefix", "");
	string phoneSetFilename = params.asString("phone_set_file", "");
	//int cvNumFolds = atoi(params.asString("cv", "0").c_str());
	//int cvOnlyDoFold = atoi(params.asString("cv_only_do_fold", "-1").c_str());
	//string filters = params.asString("filters", "3:100");
	bool stochasticPooling = (params.asString("stochastic_pooling", "false") == "true");
	//int outputDimension = atoi(params.asString("output_dimension", "10").c_str());
	double l2NormConstraint = atof(params.asString("l2_norm_constraint", "-1.0").c_str());
	string forwardLSTMs = params.asString("forward_lstms", "1:10");
	string backwardLSTMs = params.asString("backward_lstms", "0");
	string forwardGRUs = params.asString("forward_grus", "0");
	string backwardGRUs = params.asString("backward_grus", "0");
	string algorithm = params.asString("algorithm", "adadelta");
	double stepsize = atof(params.asString("stepsize", "0.1").c_str());
	int minibatchSize = atoi(params.asString("minibatch_size", "10").c_str());
	int numEpochs = atoi(params.asString("num_epochs", "3").c_str());
	int decodeEveryK = atoi(params.asString("decode_every_k", "5000").c_str());
	double softmaxDropoutRate = atof(params.asString("softmax_dropout_rate", "0.5").c_str());
	double inputDropoutRate = atof(params.asString("input_dropout_rate", "0.5").c_str());
	double hiddenDropoutRate = atof(params.asString("hidden_dropout_rate", "0.5").c_str());
	double gradientNormThreshold = atof(params.asString("gradient_norm_threshold", "1.0").c_str());
	bool useCost = (params.asString("use_cost", "false") == "true");
	bool useCoarseCost = (params.asString("use_coarse_cost", "false") == "true");

	// Change following to true to do finite difference checking of gradients.
	bool doFiniteDifferenceChecking = false;
	srand(1);

	struct timeval initialTime, endTime;
	gettimeofday(&initialTime, 0);

	vector<string> trainLines, devLines, testLines;
	readLinesFromFile(trainFilename, trainLines);
	readLinesFromFile(devFilename, devLines);
	readLinesFromFile(testFilename, testLines);

	SegmentClassifierModel scm(10000, 2000000, 10000, 60000);
	double accuracy = scm.trainAndTest(trainLines, devLines, testLines, devPredFilenamePrefix, testPredFilenamePrefix,
		phoneSetFilename,
		forwardLSTMs, backwardLSTMs,
		forwardGRUs, backwardGRUs,
		algorithm, stepsize, minibatchSize, numEpochs, decodeEveryK,
		softmaxDropoutRate, inputDropoutRate, hiddenDropoutRate,
		stochasticPooling, l2NormConstraint, gradientNormThreshold, useCost, useCoarseCost, doFiniteDifferenceChecking);

	cerr << "Final accuracy: " << accuracy << endl;

	gettimeofday(&endTime, 0);
	double cumulativeTime = computeTimeDifference(initialTime, endTime);
	cerr << "Total training time:\t" << cumulativeTime << endl;
    return 0;
}
