#include "utils.h"
#include <sstream>
#include <fstream>
#include <cstdlib>
#include <iostream>
#include <assert.h>
#include <algorithm>
#include <math.h>
#include <cstring>
#include <sys/time.h>

const double neg_inf = log(0.0);
const double pos_inf = -neg_inf;
const float neg_inf_float = log(0.0);
const float pos_inf_float = -neg_inf_float;

void tokenize(const string& str, char delimiter, vector<string>& tokens) {
	// Skip delimiters at beginning. Find first "non-delimiter".
   	string::size_type lastPos = str.find_first_not_of(delimiter, 0);
	// Find next instance of the delimiter.
   	string::size_type pos = str.find_first_of(delimiter, lastPos);

   	while (string::npos != pos || string::npos != lastPos) {
       	// Add the token from lastPos to pos.
       	tokens.push_back(str.substr(lastPos, pos - lastPos));

       	// Find the start of the next token.
       	lastPos = str.find_first_not_of(delimiter, pos);

       	// Find the end of the token.
       	pos = str.find_first_of(delimiter, lastPos);
   	}
}

double logAdd(const double lx, const double ly) {
	if (lx == neg_inf) return ly;
	if (ly == neg_inf) return lx;
	double d = lx - ly;
	if (d >= 0) {
		if (d > 50) return lx;
		else return (double)(lx + log(1.0 + exp(-d)));
	} else {
		if (d < -50) return ly;
		else return (double)(ly + log(1.0 + exp(d)));
	}
}

double logSubtract(const double lx, const double ly) {
	// if x is greater,
	if (lx > ly) {
		return (double)(lx + log(1.0 - exp(ly - lx)));
	} else {
		return (double)(ly + log(1.0 - exp(lx - ly)));
	}
}

// Converts int to string.
string itos(int i) {
	stringstream s;
	s << i;
	return s.str();
}

// Converts double to string.
string ftos(double d) {
	stringstream s;
	s << d;
	return s.str();
}







int roundToNearestInt(double x) {
  return (int)(x + 0.5);
}


double computeTimeDifference(struct timeval& time1, struct timeval& time2) {
	double secdiffDouble = (double) (time2.tv_sec - time1.tv_sec);
	long usecdiff = time2.tv_usec - time1.tv_usec;
	double scaledMicroSecDiffDouble = 0.000001 * ((double) usecdiff);
	return (secdiffDouble + scaledMicroSecDiffDouble);
}


int getRandomInteger(int maxValue) {
	double randDouble = ((double) rand()) / ((double) RAND_MAX);
	return (int)(randDouble * (((double)maxValue) + 1.0));
}


// Reads all lines from given file, placing them in the given vector.
void readLinesFromFile(const string& inputFile, vector<string>& lines) {
	ifstream inputFilestream(inputFile.c_str());
	if (!inputFilestream) {
		cerr << "Error: Cannot open file " << inputFile << "!" << endl;
		return;
	}
	string str1;
	while (!inputFilestream.eof()) {
		getline(inputFilestream, str1);
		// Note: following line should be safer:
		if (!inputFilestream.eof()) {
			lines.push_back(str1);
		}
	}
	inputFilestream.close();
}



string lowercase(const string& s) {
	string ret = s;
	for (unsigned int i = 0; i < ret.length(); ++i) {
		ret[i] = tolower(ret[i]);
	}
	return ret;
}

// From Rodrigo C Dias (http://www.codeproject.com/KB/stl/stdstringtrim.aspx)
string trimString(const string& str) {
  string::size_type pos1 = str.find_first_not_of(' ');
  string::size_type pos2 = str.find_last_not_of(' ');
  return str.substr(pos1 == string::npos ? 0 : pos1, pos2 == string::npos ? str.length() - 1 : pos2 - pos1 + 1);
}

string changeDigitsToGivenChar(const string& s, char givenChar, bool* foundDigits) {
	string ret = s;
	*foundDigits = false;
	for (unsigned int i = 0; i < ret.length(); ++i) {
		if (isdigit(ret[i])) {
			ret[i] = givenChar;
			*foundDigits = true;
		}
	}
	return ret;
}


// Returns a random double between 0 and 1.
double getRandomDouble() { return (((double) rand()) / ((double) RAND_MAX)); }

double computeVectorNorm(const vector<double>& v) {
	double sumSquares = 0.0;
	// Go through vector.
	for (vector<double>::const_iterator iter = v.begin(); iter != v.end(); ++iter) {
		double curr = *iter;
		// Add the square of the current value.
		sumSquares += curr * curr;
	}
	// Return the square root of the sum of squared values.
	return sqrt(sumSquares);
}

double standardDeviation(const vector<double>& v) {
	double mean = 0.0;
	// Compute the mean.
	for (unsigned int i = 0; i < v.size(); ++i) {
		mean += v[i];
	}
	double n = (double)(v.size());
	mean /= n;

	// Compute the squared deviations.
	double stdDev = 0.0;
	for (unsigned int i = 0; i < v.size(); ++i) {
		double currDev = v[i] - mean;
		stdDev += (currDev * currDev);
	}
	// Divide by n.
	stdDev /= n;
	// Take the square root.
	stdDev = sqrt(stdDev);
	return stdDev;
}

void spearmanCorrelation(const vector<double>& data1, const vector<double>& data2, double& d, double& rs) {
	int n = data1.size();
	double fac, en3n, en;
	double sf = 0;
	double sg = 0;
	vector<double> wksp1 = data1;
	vector<double> wksp2 = data2;

	sortTwoVectorsByFirstVector(wksp1, wksp2);
	convertSortedVectorToRanks(wksp1, sf);
	sortTwoVectorsByFirstVector(wksp2, wksp1);
	convertSortedVectorToRanks(wksp2, sg);
	d = 0;
	for (unsigned int j = 0; j < n; ++j) {
		d += pow(wksp1[j]-wksp2[j], 2);
	}
	en = n;
	en3n = en*en*en-en;
	//aved = en3n / 6.0 - (sf+ sg)/12;
	fac = (1.0 - sf/en3n) * (1.0-sg/en3n);
	if (fac > 0.0) {
		rs = (1 - (6 / en3n) * (d + (sf + sg) / 12)) / sqrt(fac);
		//fac = (rs + 1) * (1 - rs);
	} else {
		rs = 0;
	}
}


// Given a sorted array w[0..n-1], replaces the elements by their rank, including midranking
// of ties and returns as s the sum of f^3-f, where f is the number of element in each tie.
void convertSortedVectorToRanks(vector<double>& w, double& s) {
	int j = 1;
	int ji, jt;
	double t, rank;
	int n = w.size();
	s = 0;

	while (j < n) {
		if (w[j] != w[j-1]) {
			w[j-1] = j;
			++j;
		} else {
			for (jt = j + 1; jt <= n && w[jt-1] == w[j-1]; jt++);
			rank = 0.5 * (j + jt - 1);
			for (ji = j; ji <= (jt-1); ji++) w[ji-1] = rank;
			t = jt - j;
			s += (t*t*t-t);
			j = jt;
		}
	}
	if (j == n) w[n - 1] = n;
}

void sortTwoVectorsByFirstVector(vector<double>& arr, vector<double>& brr) {
	vector<pair<double, double> > pairs;
	if (arr.size() != brr.size()) {
		cerr << "Error: when comparing two vectors, sizes differ! arr.size() = " << arr.size() << ", brr.size() = " << brr.size() << endl;
	}
	pairs.reserve(arr.size());
	for (unsigned int i = 0; i < arr.size(); ++i) {
		pairs.push_back(make_pair(arr[i], brr[i]));
	}
	std::sort(pairs.begin(), pairs.end());//, CompareDoublePairVectorFirstDouble);

	for (unsigned int i = 0; i < arr.size(); ++i) {
		arr[i] = pairs[i].first;
		brr[i] = pairs[i].second;
	}
}
